@@ -1,4 +1,4 @@
-import platform, os, ctypes, pkg_resources, sys
+import platform, os, ctypes, pkg_resources, sys, gc
 import subprocess
 import numpy as np
 
@@ -53,9 +53,20 @@ class C_functions:
             del self._cdll
         else:
             ctypes.cdll.LoadLibrary('libdl.so').dlclose(self._cdll._handle)
+        gc.collect()
         os.remove(self._path)
         
         
+class cuda_functions:
+    def __init__(self, func):
+        self._func = func
+        self._block, self._thread = None, None
+    
+    def __getitem__(self, item):
+        temp = self.__class__(self._func)
+        temp._block, temp._thread = item
+        return temp
+
 
 
 # c and python interface.
@@ -73,7 +84,6 @@ c_to_py ={
     'long*'      : np.ctypeslib.ndpointer(dtype=np.int64),
     'double*'   : np.ctypeslib.ndpointer(dtype=np.float64),
 
-    'int**'     : np.ctypeslib.ndpointer(dtype=np.float64, ndim=2),
     'void'      : None
 }
 
@@ -224,11 +234,14 @@ def function_def_check(file):
 
 
 def precompile(filename, tempname, encoding = 'utf8', debug = False):
+    cu = (os.path.splitext(filename)[1].lower() == "cu")
     cpp = not (os.path.splitext(filename)[1].lower() == "c")
     with  open(filename, encoding=encoding) as cppsource:
         header = []
         functions = []
+        cu_functions = []
         docs = []
+        cu_docs = []
         header_line = -1
         defs = False
         defse = False
@@ -240,12 +253,18 @@ def precompile(filename, tempname, encoding = 'utf8', debug = False):
                 doc, funcdef  = function_def_check(cppsource)
                 docs.append(doc)
                 functions.append(")".join(funcdef.split(")")[:-1])+");\r\n")
-            if line[:13] == "//@python_var":
-                functions.append(cppsource.readline())
+            if line[:18] == "//@python_cuda_def":
+                if not cu:
+                    raise NotImplementedError("python_cuda_def method is only for .cu file!")
+                doc, funcdef  = function_def_check(cppsource)
+                cu_docs.append(doc)
+                cu_functions.append(")".join(funcdef.split(")")[:-1])+");\r\n")
         if header_line == -1:
             raise SyntaxError("Cannot find '//@extern_python' from source.")
 
+    #C functions organizing
     func_dict = {}
+    cu_func_dict = {}
     doc_dict = {}
     if debug:
         print("C functions found from source : ")
@@ -255,7 +274,15 @@ def precompile(filename, tempname, encoding = 'utf8', debug = False):
         doc_dict[name] = docstring
         if debug:
             print(f"\t{name}")
-        
+    #cuda functions organizing
+    if debug:
+        print("Cuda functions found from source : ")
+    for docstring, func in zip(cu_docs, cu_functions):
+        name, types = cdefs_analyzer(func)
+        cu_func_dict[name] = types
+        doc_dict[name] = docstring
+        if debug:
+            print(f"\t{name}")
 
     #make new C source for python 
     with open(tempname,"w") as cpp_source:
@@ -266,8 +293,12 @@ def precompile(filename, tempname, encoding = 'utf8', debug = False):
                     if func_name in func_dict:
                         continue
                     else:
+                        if func_name in cu_func_dict:
+                            continue
                         cpp_source.write(line)
                 elif i == header_line:
+                    for cudefs in cu_functions:
+                        cpp_source.write(cudefs)
                     if cpp:
                         cpp_source.write('extern "C"{\r\n')
                     for cdefs in functions:
@@ -275,6 +306,24 @@ def precompile(filename, tempname, encoding = 'utf8', debug = False):
                             cpp_source.write("__declspec(dllexport) "+cdefs)
                         else:
                             cpp_source.write(cdefs)
+                    for cudefs in cu_functions:
+                        cufunc = cudefs.split("(")
+                        cufunc[0] = cufunc[0].rstrip() + "_py"
+                        cufunc[0] = cufunc[0].split()[-2:]
+                        cufunc[1] = "int* ___block, int* ___thread, " + cufunc[1]    ##### ___ 3"_" block and thread must be length 3 array always
+                        newdef = ' ('.join(cufunc)                   ## cufunc : __device__ (restype) some_func(args) -> newdef : (restype) some_func_py(args)
+                        ")".join(newdef.split(")")[:-1])
+                        cpp_source.write(newdef)  ###################  declaration
+                        cpp_source.write("""{
+                            dim3 tpb (___thread[0],___thread[1],___thread[2])
+                            dim3 bpg (___block[0],___block[1],___block[2])
+                            """)
+
+                        cufunc = cudefs.split("(")
+                        cufunc[0] += "<<<bpg, tpb>>>"
+                        cpp_source.write("(".join(cufunc))
+                        cpp_source.write
+                        pass ### cuda function wrapper
                     if cpp:
                         cpp_source.write('};\r\n')
                         
@@ -283,6 +332,8 @@ def precompile(filename, tempname, encoding = 'utf8', debug = False):
                     cpp_source.write(line)
     return functions, doc_dict, cpp
 
+
+
 def compile(filename, outputname, *flags, compiler=None, cpp = False, message= True):
     """compile give C(C++) code with option in flags.
     Parameters
@@ -363,7 +414,7 @@ def compile_option_parser(filename, encoding = "utf8"):
 
 
 
-def from_C_source(filename, *flags, debug = False, encoding = 'utf8', output= None):
+def from_C_source(filename, *flags, debug = False, encoding = 'utf8', compiler= None):
     tempname = "__temp__"+os.path.basename(filename)
     libname = os.path.splitext(os.path.basename(filename))[0]
 
