@@ -19,27 +19,30 @@ class IsingModelSingle:
     algorithm : `str`
         specifying Monte Carlo algorithm. 'wolff' and 'metropolis' can be captured. Default is 'metropolis'
     '''
-    _cdll = internal_Library('isingmonte')
-    metropolis = _cdll.monteCarlo
-    metropolis.argtypes = [np.ctypeslib.ndpointer(dtype=np.int32),
-                 np.ctypeslib.ndpointer(dtype=np.int32),
-                 ctypes.c_double, 
-                 ctypes.c_int,
-                ctypes.c_int,
-                ctypes.c_int,
-                ctypes.c_int,
-                ctypes.c_int]
-    wolff = _cdll.wolff
-    wolff.argtypes = [np.ctypeslib.ndpointer(dtype=np.int32),
-                 np.ctypeslib.ndpointer(dtype=np.int32),
-                 ctypes.c_double, 
-                 ctypes.c_int,
-                ctypes.c_int,
-                ctypes.c_int,
-                ctypes.c_int,
-                ctypes.c_int]
-    
-    _modules = {'np':'numpy'}
+    _cdll = None
+    def _load_cdll(self):
+        if self.__class__._cdll is None:
+            self.__class__._cdll = internal_Library('isingmonte')
+            self.__class__.metropolis = self.__class__._cdll.monteCarlo
+            self.__class__.metropolis.argtypes = [np.ctypeslib.ndpointer(dtype=np.int32),
+                        np.ctypeslib.ndpointer(dtype=np.int32),
+                        ctypes.c_double, 
+                        ctypes.c_int,
+                        ctypes.c_int,
+                        ctypes.c_int,
+                        ctypes.c_int,
+                        ctypes.c_int]
+            self.__class__.wolff = self.__class__._cdll.wolff
+            self.__class__.wolff.argtypes = [np.ctypeslib.ndpointer(dtype=np.int32),
+                        np.ctypeslib.ndpointer(dtype=np.int32),
+                        ctypes.c_double, 
+                        ctypes.c_int,
+                        ctypes.c_int,
+                        ctypes.c_int,
+                        ctypes.c_int,
+                        ctypes.c_int]
+            
+            self.__class__._modules = {'np':'numpy'}
     
     
     def __init__(self, L, algorithm = 'wolff'):
@@ -51,9 +54,10 @@ class IsingModelSingle:
         self.algorithm = 'wolff'
         if not algorithm in ['wolff', 'metropolis']:
             raise ValueError("Wrong algorithm, only 'wolff' and 'metropolis' are allowed. ")
+        self._load_cdll()
             
     def __repr__(self):
-        return f"<Single ising model simulator>\nL\t\t : {self.L}\n" + f"T\t\t : {self.T_range}\n"+f"MC_steps\t : {self.simulate_time}"
+        return f"<Single ising model simulator>\nL\t\t : {self.L}\n"+f"T\t\t : {self.T_range}\n"+f"MC_steps\t : {self.simulate_time}"
         
     def __gt__(self, other):
         return self.L > other.L
@@ -234,56 +238,6 @@ class IsingModel:
         return temp
         
 
-class IsingSingleAnalyzer:
-    _modules = {'np':'numpy'}
-    def __init__(self, L, T, E, M):
-        self.L = L
-        self.T = T
-        self.E = E
-        self.M = np.abs(M)
-        
-        self.total_ensemble = None
-        self.sim_time       = None
-        if isinstance(E, np.ndarray):
-            assert E.shape == M.shape
-            assert E.shape[0]==T.shape[0]
-            assert M.shape[0]==T.shape[0]
-
-            self.total_ensemble = E.shape[1]
-            self.sim_time = E.shape[2]
-            
-        self.__analyzed = False
-    
-    def analyze(self, reduced = False):
-        if self.__analyzed: return
-        self.average  = Container()
-        self.var      = Container()
-        self.second   = Container()
-        self.forth    = Container()
-        
-        #self.meaned   = Container()
-        
-        for key in vars(self).copy():
-            if isinstance(vars(self)[key], np.ndarray) and len(vars(self)[key].shape)==3:
-                vars(self.average)[key] = np.average(vars(self)[key], axis =2)
-                vars(self.var)[key]     = np.var(vars(self)[key], axis =2)
-                vars(self.second)[key]  = np.average(vars(self)[key].astype(np.float64)**2, axis =2)
-                vars(self.forth)[key]   = np.average(vars(self)[key].astype(np.float64)**4, axis =2)
-            if isinstance(vars(self)[key], list) and len(vars(self)[key][0].shape)==2:
-                temp = [[],[],[],[]]
-                for i in range(len(vars(self)[key])):
-                    temp[0].append(np.average(vars(self)[key][i], axis =1))
-                    temp[1].append(np.var(vars(self)[key][i], axis =1))
-                    temp[2].append(np.average(vars(self)[key][i].astype(np.float64)**2, axis =1))
-                    temp[3].append(np.average(vars(self)[key][i].astype(np.float64)**4, axis =1))
-                vars(self.average)[key], vars(self.var)[key], vars(self.second)[key], vars(self.forth)[key] = np.array(temp)
-        
-        if reduced:
-            del self.E, self.M
-                
-        self.__analyzed = True
-                
-
 class Container(object):
     pass
 
@@ -298,7 +252,7 @@ class Observable:
     @average.getter
     def average(self):
         if self.__aver is None:
-            self.__aver = np.average(obj, axis = 2)
+            self.__aver = np.average(self.__raw, axis = 2)
     
 
 class IsingSingleAnalyzer:
@@ -306,7 +260,8 @@ class IsingSingleAnalyzer:
         self.L = L
         self.T = T
         self.E = E
-        self.M = np.abs(M)
+        self.M = M
+        self.O = np.abs(M)
         
         assert E.shape == M.shape
         assert E.shape[0]==T.shape[0]
@@ -354,7 +309,7 @@ class IsingSingleAnalyzer:
     
     @observable
     def susceptibility(self):
-        return self.var.M/self.L/self.L/self.T
+        return self.var.O/self.L/self.L/self.T
     
     @observable
     def heat_capacity(self):
@@ -362,14 +317,14 @@ class IsingSingleAnalyzer:
     
     @observable
     def binder_cumulant(self):
-        forth = self.forth.M
-        second = self.second.M
+        forth = self.forth.O
+        second = self.second.O
         return 1 - forth/3/second**2
         
                 
                 
         
-class IsingMultiAnalyzer:
+'''class IsingMultiAnalyzer:
     def __init__(self,L,T,E,M, title = ""):
         self.entry = []
         self.L = L
@@ -473,7 +428,7 @@ class IsingMultiAnalyzer:
             yield isa
 
     def __getitem__(self, value):
-        return self.entry[value]
+        return self.entry[value]'''
         
 def ens_mean(x):
     return np.average(x, axis=1)
